#include "cpu_config.h"

// x8 + x4 + x3
#define CRC_POLY 0x8C  

#define CHANELS true
#define MULTISWITCH false

const uint8_t ChStartByte = 1;	// chanels
const uint8_t MsStartByte = 2;	// multiswitch
const uint8_t EndByte = 3;   
   
#define FRAMESIZE 11                      //  StartByte CH1...CH8 CRC EndByte
#define DATASIZE (FRAMESIZE - 2)          //  CH1...CH8 CRC
volatile uint8_t inUart_arr[DATASIZE];    // low level buffer CH1...CH8 CRC
volatile uint8_t receiv_arr[DATASIZE-1];  // valid data buffer CH1...CH8
volatile uint8_t BYTEinCount = 0;
volatile boolean NewFrame = false;
volatile boolean ChFrame  = true;         // true chanels false sw chanels

unsigned long prev = 0;                   // for time display

// cyclic redundance code
uint8_t CRC8(uint8_t *addr, uint8_t len)
{
	uint8_t crc = 0;
	
	for (uint8_t i=0; i<len;i++)
		{
			uint8_t inbyte = addr[i];
			for (uint8_t j=0;j<8;j++)
			{
				uint8_t mix = (crc ^ inbyte) & 0x01;
				crc >>= 1;
				if (mix)
					crc ^= CRC_POLY;

				inbyte >>= 1;
			}
		}

	return crc;	
}

// serial reception ISR
ISR (BYTEin) {										  
	
	uint8_t	dat = 0;								
	dat = UDRn;

  if(NewFrame) return;

	if (dat == ChStartByte)
	{	
		ChFrame  = true;
		BYTEinCount = 0;		
		for (uint8_t i = 0; i < DATASIZE; i++) inUart_arr[i] = 0;
		return;
	}
	
	if (dat == MsStartByte)
	{	
		ChFrame  = false;
		BYTEinCount = 0;		
		for (uint8_t i = 0; i < DATASIZE; i++) inUart_arr[i] = 0;
		return;
	}
		
	if (dat == EndByte)
	{
		NewFrame = true;
		return;
	}

	if(BYTEinCount < DATASIZE)
	{ 
		inUart_arr[BYTEinCount] = dat;
		BYTEinCount++;
	}
}

// CPU initialisation for wireless serial transmition
void startReceiver (uint32_t baudRate) {
//uart baud rate
uint16_t ubrr = 0;
if (baudRate == 115200) ubrr = 8;
else ubrr = 16000000/16/baudRate-1;
UBRRnH = (unsigned char)(ubrr>>8);
UBRRnL = (unsigned char)ubrr;

//   frame format: asynchrone, no parity, 8 data bits, 1 stop bit 
SetBit(UCSRnC, 1);  // 1 << 1 = `0000 0010`
SetBit(UCSRnC, 2);  // 1 << 2 = `0000 0100`

// enable Tx Rx & Rx interrupt
SetBit(UCSRnB, TXEN0);  
SetBit(UCSRnB, RXEN0);       
SetBit(UCSRnB, RXCIE0);

// init buf for neutral servo position 1.5 ms
for (uint8_t i = 0; i < 8; i++) receiv_arr[i] = 127; 
}



	
	
