#include "Arduino.h"

#define  SetBit(reg, bita)         reg |= (1<<bita)            
#define  ClearBit(reg, bita)       reg &= (~(1<<bita))
#define  InvBit(reg, bita)         reg ^= (1<<bita)
#define  BitIsSet(reg, bita)       ((reg & (1<<bita)) != 0)
#define  BitIsClear(reg, bita)     ((reg & (1<<bita)) == 0)

// Arduino Duemilanove, Diecimila, and NG
#if defined(__AVR_ATmega168__) || defined(__AVR_ATmega328P__) || defined(__AVR_ATmega328__)

#define LedPinOut() DDRB  |= (1<<5)
#define LedPinOn()  PORTB |= (1<<5)
#define LedPinOFF() PORTB &= (~(1<<5))

// Arduino Mega
#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)

#define LedPinOut() DDRB  |= (1<<7)
#define LedPinOn()  PORTB |= (1<<7)
#define LedPinOFF() PORTB &= (~(1<<7))

// Leonardo
#elif defined(__AVR_ATmega32U4__)

#define LedPinOut() DDRC  |= (1<<7)
#define LedPinOn()  PORTC |= (1<<7)
#define LedPinOFF() PORTC &= (~(1<<7))

// anything else
#else
#error "Board not supported"
#endif

#if defined(Serial_0) 			//UART0

#if defined(UBRRH) && defined(UBRRL) 
	#define  UDRn     UDR
	#define  UBRRnH   UBRRH
	#define  UBRRnL   UBRRL
	#define  UCSRnA   UCSRA 
	#define  UCSRnB   UCSRB  
	#define  UCSRnC   UCSRC      
#else
	#define  UDRn     UDR0
	#define  UBRRnH   UBRR0H
	#define  UBRRnL   UBRR0L
	#define  UCSRnA   UCSR0A 
	#define  UCSRnB   UCSR0B  
	#define  UCSRnC   UCSR0C
#endif

#if defined(USART_RX_vect)
  #define BYTEin USART_RX_vect  
#elif defined(USART0_RX_vect)
  #define BYTEin USART0_RX_vect
#elif defined(USART_RXC_vect)
  #define BYTEin USART_RXC_vect //ATmega8
#else
  #error "Board not supported"
#endif

#endif  			  			//UART0


#if defined(Serial_1) 			//UART1

	#define  UDRn     UDR1
	#define  UBRRnH   UBRR1H
	#define  UBRRnL   UBRR1L
	#define  UCSRnA   UCSR1A 
	#define  UCSRnB   UCSR1B  
	#define  UCSRnC   UCSR1C 
	
	#if defined(UART1_RX_vect)
		#define	BYTEin UART1_RX_vect
	#elif defined(USART1_RX_vect)
		#define BYTEin USART1_RX_vect
	#else
	#error "Board not supported"
	#endif
		
#endif							//UART1


#if defined(Serial_2)			//UART2

	#define  UDRn     UDR2
	#define  UBRRnH   UBRR2H
	#define  UBRRnL   UBRR2L  
	#define  UCSRnA   UCSR2A 
	#define  UCSRnB   UCSR2B  
	#define  UCSRnC   UCSR2C 
	#define  BYTEin   USART2_RX_vect
			
#endif							//UART2


#if defined(Serial_3)			//UART3

	#define  UDRn     UDR3
	#define  UBRRnH   UBRR3H
	#define  UBRRnL   UBRR3L
	#define  UCSRnA   UCSR3A 
	#define  UCSRnB   UCSR3B  
	#define  UCSRnC   UCSR3C 
	#define  BYTEin   USART3_RX_vect
			
#endif							//UART3

#if !defined(TXC0)
#if defined(TXC)
// Some chips like ATmega8 don't have UPE, only PE. The other bits are
// named as expected.
#if !defined(UPE) && defined(PE)
#define UPE PE
#endif
// On ATmega8, the uart and its bits are not numbered, so there is no TXC0 etc.
#define TXC0 TXC
#define RXEN0 RXEN
#define TXEN0 TXEN
#define RXCIE0 RXCIE
#define UDRIE0 UDRIE
#define U2X0 U2X
#define UPE0 UPE
#define UDRE0 UDRE
#elif defined(TXC1)
// Some devices have uart1 but no uart0
#define TXC0 TXC1
#define RXEN0 RXEN1
#define TXEN0 TXEN1
#define RXCIE0 RXCIE1
#define UDRIE0 UDRIE1
#define U2X0 U2X1
#define UPE0 UPE1
#define UDRE0 UDRE1
#else
#error No UART found in HardwareSerial.cpp
#endif
#endif // !defined TXC0

