/*
Timer1 is loaded with a pulse width value.
ISR (T1 overflow interrupt) writes a high level on digital output chanel
for pulse width time, then put it low and writes the next chanel
Ch1 = digital 3 (pin D3 mega2530) .... CH8 = D10
*/

volatile uint8_t tidx = 0; 		// Chanel buf pointer
const uint8_t pinBuf[8] = {3,4,5,6,7,8,9,10};


ISR (TIMER1_OVF_vect)    // Timer1 overflow ISR
{
	digitalWrite(pinBuf[tidx], LOW); 
	if(tidx == 7)
		tidx = 0;
	else
		tidx++;
	TCNT1 = 65535 - (uint16_t) (receiv_arr[tidx] + 173)*10;  
	digitalWrite(pinBuf[tidx], HIGH); 
	
}


void timer_init()
{
	// Timer1 16bits for pulse mesurement CPU Freq 16Mhz
	// for interval of 1ms ==> 0,001/(1/16000000) = 16.000 ticks
	// Prescaler 8 ==> resolution changes from 0,0000000625 to 0,0000005 = 0.5 uS
	// Need interval of 3Ms ==> 0,003/((1/16000000)*8) = 6000 ticks 0x1770
	// Set prescaler to 1/8 ; (1 << CS31)  (p. 162)
	
	TCCR1A = 0;  
	TCCR1B = 0x02;
	
	tidx = 0;
	TCNT1 = 65535-3000;	 // init OVF to 1.5 ms
    TIMSK1 = (1 << TOIE1) ;   // Enable timer1 overflow interrupt(TOIE1)
	sei();        // Enable global interrupts

}
