#include "cpu_config.h"


// x8 + x4 + x3
#define CRC_POLY 0x8C  

const uint8_t ChStartByte = 1;		// chanels
const uint8_t MsStartByte   = 2;	// multiswitch
const uint8_t EndByte   = 3;   

byte transm_arr[11]; // StartByte CH1 ... CH8 CRC  EndByte


unsigned long prev = 0;
unsigned long my_delay = 0;

uint8_t CRC8(uint8_t *addr, uint8_t len)
{
	uint8_t crc = 0;
	
	for (uint8_t i=0; i<len;i++)
		{
			uint8_t inbyte = addr[i];
			for (uint8_t j=0;j<8;j++)
			{
				uint8_t mix = (crc ^ inbyte) & 0x01;
				crc >>= 1;
				if (mix)
					crc ^= CRC_POLY;

				inbyte >>= 1;
			}
		}

	return crc;	
}

/*
servo pulse range (see PulsRange.xlsx)
pulse	885 uS  1500 uS  2137 uS
tx_buf    4      127      254
*/
boolean load_data(uint8_t target) {	
uint8_t i = 0, j = 0, k = 0;
boolean flag = false;

	switch(target)
	{
		case STDCH:
			transm_arr[0] = ChStartByte;
			
			for(i=0; i < 9; i++)
			{
				if(tmp_buf[i] >= TSYNC) 
				{
					flag = true;
					break;
				}		
			}
			
			if(flag)
			{
				for(j=i+1; j<9; j++)
				{
					if(tmp_buf[j] > 4270) 
						tx_buf[k] = 255; // 4270/10 - 173 = 254
					else
						if(tmp_buf[j] < 1770) 
							tx_buf[k] = 4;	// 1770/10 - 173 = 4					
						else
							tx_buf[k] = (uint8_t) (tmp_buf[j]/10 -173);
					k++;
				}
				
				for(j=0; j<i; j++)
				{
					if(tmp_buf[j] > 4270) 
						tx_buf[k] = 255; 
					else
						if(tmp_buf[j] < 1770) 
							tx_buf[k] = 4;					
						else
							tx_buf[k] = (uint8_t) (tmp_buf[j]/10 -173);
					k++;
				}
				
			}
			else
				return false;
			break;
		
		
		case SWCH:
			transm_arr[0] = MsStartByte;
			for(i=0; i < 9; i++)
			{
				if(tmp_buf[i] <= TSYNCM) 
				{
					flag = true;
					break;
				}		
			}
			if(flag)
			{
				for(j=i+1; j<9; j++)
				{
					if(tmp_buf[j] > 4270) 
						tx_buf[k] = 255; 
					else
						if(tmp_buf[j] < 1770) 
							tx_buf[k] = 4;						
						else
							tx_buf[k] = (uint8_t) (tmp_buf[j]/10 -173);
					k++;
				}
				
				for(j=0; j<i; j++)
				{
					if(tmp_buf[j] > 4270) 
						tx_buf[k] = 255; 
					else
						if(tmp_buf[j] < 1770) 
							tx_buf[k] = 4;					
						else
							tx_buf[k] = (uint8_t) (tmp_buf[j]/10 -173);
					k++;
				}
				
			}
			else
				return false;
			break;
	}
  
  for (i = 1; i < 9; i++) 
		transm_arr[i] = tx_buf[i-1];
  
  transm_arr[9] = CRC8(transm_arr + 1,8);
  transm_arr[10] = EndByte;
  return true;
}


// NEVER CALL transmit() FROM an ISR() or DEAD LOCK !
// 1 start byte + 8 data bytes + 1 CRC byte  + 1 stop byte = 11 
// 9600 baudrate  1 start bit 8 data bit  no parity 1 stop bit = 10 bits for 1 data byte
// https://lucidar.me/fr/serialib/convert-bauds-to-bits-per-second/
// -> 960 bytes/s -> 10^3/960 = 1.04 byte/ms 
// so can send 22 bytes max during a ppm cycle = 22.5 ms

void transmit(void){
for (uint8_t i = 0; i < 11; i++) 
    {
      while ( !( UCSRnA & (1<<UDRE0)) );  // wait for empty buffer
      UDRn = transm_arr[i];
    } 		
}

void display(unsigned char item)
{
  switch(item)
  {
    case CH:
       Serial.print("PPM ");
       Serial.println(my_delay);
     /*  
       for (uint8_t i=1; i<8;i++)
        { Serial.print(transm_arr[i]);
          Serial.print("  ");
        }    
      Serial.println(transm_arr[8]);
      */
    break;

    case SW:
      Serial.print("SW ");
      Serial.println(my_delay);
      /*
      for (uint8_t i=1; i<8;i++)
        { Serial.print(transm_arr[i]);
          Serial.print("  ");
        }    
      Serial.println(transm_arr[8]);
      */
    break;
    
    case BCH:       
       Serial.print("PPM BUF  ");
       Serial.println(my_delay);
       for (uint8_t i=0; i<8;i++)
        { Serial.print(tmp_buf[i]);
          Serial.print("  ");
        }    
      Serial.println(tmp_buf[8]);
    break;

    case BSW:
      Serial.print("SW BUF  ");
      Serial.println(my_delay);
      for (uint8_t i=0; i<8;i++)
        { Serial.print(tmp_buf[i]);
          Serial.print("  ");
        }    
      Serial.println(tmp_buf[8]);
    break;
  }
  

}


void startTransmitter (uint32_t baudRate) {
// doc p 211
// uart baud rate
uint16_t ubrr = 0;
if (baudRate == 115200) ubrr = 8;
else ubrr = 16000000/16/baudRate-1;
UBRRnH = (unsigned char)(ubrr>>8);
UBRRnL = (unsigned char)ubrr;

// frame format  asynchrone no parity 8 data 1 stop bit (doc p 225)
SetBit(UCSRnC, 1);
SetBit(UCSRnC, 2);
 
// enable Tx Rx (transceiver mode)
SetBit(UCSRnB, TXEN0);
SetBit(UCSRnB, RXEN0);
//  no rx interrupt   (transmitter mode)  
//SetBit(UCSRnB, RXCIE0);

//Serial6bSendTimeOUT = timeSend; 

// possibly Timer0 interrupt for synchronisation
	// CPU Freq 16Mhz
	// Need interval of 1Ms ==> 0,001/(1/16000000) = 16.000 ticks
	// Prescaler 64 ==> resolution changes from 0,0000000625 to 0,000004 = 4 uS
	// Need interval of 1Ms ==> 0,001/((1/16000000)*64) = 250 ticks
	// Set prescaler to 64 ; (1 << CS01)|(1 << CS00)
	// Clear Timer on Compare (CTC) mode ; (1 << WGM02)
//TCCR0A = 0 ;   
//TCCR0B |= (1 << WGM02)|(1 << CS01)|(1 << CS00) ;

	//  T0 isr 1 ms (set Output Compare Register to (250 - 1) ticks)
//OCR0A = 0xF9;			
	// initialize counter
//TCNT0 = 0 ;	
	// Set Timer Interrupt Mask Register to
    // clear Timer on Compare channel A for timer 0
//SetBit(TIMSK0, OCIE0A);	 
						
//   PIN 13
//LedPinOut();
//  
for (uint8_t i = 0; i < 11; i++) transm_arr[i] = 0;

// general interrupts enable					
interrupts();

}
